@@ -110,6 +110,13 @@ static struct poptOption popt_options[] =
 	{ NULL, 0, 0, NULL, 0, NULL, NULL }
 };
 
+static gboolean
+unref_when_idle (gpointer user_data)
+{
+	g_object_unref (user_data);
+	return FALSE;
+}
+
 int
 main (int argc, char *argv[])
 {
@@ -119,7 +126,7 @@ main (int argc, char *argv[])
 	
 #ifdef ENABLE_NLS
 	/* Initialize the i18n stuff */
-	bindtextdomain(GETTEXT_PACKAGE, PACKAGE_LOCALE_DIR);
+	bindtextdomain(GETTEXT_PACKAGE, DATADIR "/locale");
 	bind_textdomain_codeset(GETTEXT_PACKAGE, "UTF-8");
 	textdomain(GETTEXT_PACKAGE);
 #endif
@@ -146,19 +153,30 @@ main (int argc, char *argv[])
 		
 		glade_gnome_init ();
 		
-		gnome_window_icon_set_default_from_file (PIXMAP_DIR "/galeon.png");
+		gnome_window_icon_set_default_from_file (DATADIR "/pixmaps/galeon.png");
 		
 		galeon_shell_new ();
 		
 		galeon_main_start ();
+
+		/* Flush any pending GTK events, this works around crashes when
+		 * it takes a long time to startup. */
+		while (gtk_events_pending())
+			gtk_main_iteration();
 		
 		galeon_automation_complete_initialization ();
 		
-		g_object_unref (G_OBJECT(galeon_shell));
+		g_idle_add (unref_when_idle, galeon_shell);
 		
 		bonobo_main ();
 		
 		gnome_vfs_shutdown ();
+	} 
+	else
+	{
+#if GTK_CHECK_VERSION(2,2,0)
+		gdk_notify_startup_complete ();
+#endif 
 	}
 
 	return 0;
@@ -263,10 +281,6 @@ galeon_main_start (void)
 	}
 
 	CORBA_exception_free (&corba_env);
-
-#if GTK_CHECK_VERSION(2,2,0)
-	gdk_notify_startup_complete ();
-#endif 
 }
 
 static gboolean
