@@ -73,6 +73,7 @@ struct PdmDialogPrivate
 	PdmActionInfo *password_sites;
 	PdmActionInfo *cookie_sites;
 	PdmActionInfo *image_sites;
+	PdmActionInfo *popup_sites;
 };
 
 enum
@@ -94,6 +95,9 @@ enum
 
 	PROP_PASSWORD_SITES_TREEVIEW,
 	PROP_PASSWORD_SITES_REMOVE,
+
+	PROP_POPUP_SITES_TREEVIEW,
+	PROP_POPUP_SITES_REMOVE
 };
 
 enum
@@ -131,6 +135,13 @@ enum
 	COL_PASSWORD_SITES_DATA
 };
 
+enum
+{
+	COL_POPUP_SITES_DOMAIN,
+	COL_POPUP_SITES_STATUS,
+	COL_POPUP_SITES_DATA
+};
+
 static const
 GaleonDialogProperty properties [] =
 {
@@ -152,6 +163,9 @@ GaleonDialogProperty properties [] =
 	{ PROP_PASSWORD_SITES_TREEVIEW, "password_sites_treeview", NULL, PT_NORMAL, NULL },
 	{ PROP_PASSWORD_SITES_REMOVE, "password_sites_remove_button", NULL, PT_NORMAL, NULL },
 
+	{ PROP_POPUP_SITES_TREEVIEW, "popup_sites_treeview", NULL, PT_NORMAL, NULL },
+	{ PROP_POPUP_SITES_REMOVE, "popup_sites_remove_button", NULL, PT_NORMAL, NULL },
+
 	{ -1, NULL, NULL }
 };
 
@@ -509,6 +523,61 @@ setup_image_sites_treeview (PdmDialog *dialog)
 	return GTK_WIDGET(treeview);
 }
 
+static GtkWidget *
+setup_popup_sites_treeview (PdmDialog *dialog)
+{
+	GtkTreeView *treeview;
+        GtkListStore *liststore;
+        GtkCellRenderer *renderer;
+        GtkTreeViewColumn *column;
+	GtkTreeSelection *selection;
+	
+	treeview = GTK_TREE_VIEW(galeon_dialog_get_control 
+				 (GALEON_DIALOG(dialog),
+				 PROP_POPUP_SITES_TREEVIEW));
+
+	
+	
+        /* set tree model */
+        liststore = gtk_list_store_new(3,
+                                       G_TYPE_STRING,
+                                       G_TYPE_STRING,
+                                       G_TYPE_POINTER);
+        gtk_tree_view_set_model(treeview, GTK_TREE_MODEL(liststore));
+        gtk_tree_view_set_headers_visible(treeview, TRUE);
+	selection = gtk_tree_view_get_selection(treeview);
+	gtk_tree_selection_set_mode(selection,
+				    GTK_SELECTION_MULTIPLE);
+
+	renderer = gtk_cell_renderer_text_new();
+
+	gtk_tree_view_insert_column_with_attributes(treeview,
+                                                    COL_POPUP_SITES_DOMAIN, 
+						    _("Domain"),
+                                                    renderer,
+                                                    "text", COL_POPUP_SITES_DOMAIN,
+                                                    NULL);
+        column = gtk_tree_view_get_column(treeview, COL_POPUP_SITES_DOMAIN);
+        gtk_tree_view_column_set_resizable(column, TRUE);
+        gtk_tree_view_column_set_reorderable(column, TRUE);
+        gtk_tree_view_column_set_sizing(column, GTK_TREE_VIEW_COLUMN_AUTOSIZE);
+        gtk_tree_view_column_set_sort_column_id(column, COL_POPUP_SITES_DOMAIN);
+	
+        gtk_tree_view_insert_column_with_attributes(treeview,
+                                                    COL_POPUP_SITES_STATUS, 
+						    _("Status"),
+                                                    renderer,
+                                                    "text", COL_POPUP_SITES_STATUS,
+                                                    NULL);
+        column = gtk_tree_view_get_column(treeview, COL_POPUP_SITES_STATUS);
+        gtk_tree_view_column_set_resizable(column, TRUE);
+        gtk_tree_view_column_set_reorderable(column, TRUE);
+        gtk_tree_view_column_set_sizing(column, GTK_TREE_VIEW_COLUMN_AUTOSIZE);
+        gtk_tree_view_column_set_sort_column_id(column, COL_POPUP_SITES_STATUS);
+
+	return GTK_WIDGET(treeview);
+}
+
 static void
 pdm_dialog_remove_button_clicked_cb (GtkWidget *button,
 				     PdmActionInfo *action)
@@ -681,6 +750,26 @@ pdm_dialog_image_site_add (PdmActionInfo *info,
                            -1);
 }
 
+static void
+pdm_dialog_popup_site_add(PdmActionInfo *info, 
+			  gpointer popup_site)
+{
+	GtkListStore *store;
+	GtkTreeIter iter;
+	PermissionInfo *pinfo = (PermissionInfo *)popup_site;
+
+	store = GTK_LIST_STORE(gtk_tree_view_get_model 
+			       (GTK_TREE_VIEW(info->treeview)));
+
+	gtk_list_store_append(store, &iter);
+        gtk_list_store_set(store,
+                           &iter,
+                           COL_POPUP_SITES_DOMAIN, pinfo->domain,
+                           COL_POPUP_SITES_STATUS, _("Can show popups"),
+			   COL_POPUP_SITES_DATA, pinfo,
+                           -1);
+}
+
 static void
 pdm_dialog_cookie_remove (PdmActionInfo *info,
 			  GList *data)
@@ -726,6 +815,15 @@ pdm_dialog_image_site_remove(PdmActionInfo *info,
 	galeon_embed_shell_remove_permissions(shell, IMAGES_PERMISSION, data);
 }
 
+static void
+pdm_dialog_popup_site_remove(PdmActionInfo *info,
+			     GList *data)
+{
+	GaleonEmbedShell *shell;
+	shell = galeon_shell_get_embed_shell(galeon_shell);
+	galeon_embed_shell_remove_permissions(shell, POPUPS_PERMISSION, data);
+}
+
 static void
 pdm_dialog_cookies_free (PdmActionInfo *info,
 			 GList *data)
@@ -776,6 +874,7 @@ group_button_allocations (GaleonDialog *dialog)
 		PROP_COOKIE_SITES_REMOVE,
 		PROP_IMAGE_SITES_REMOVE,
 		PROP_PASSWORD_SITES_REMOVE,
+		PROP_POPUP_SITES_REMOVE
 	};
 	GtkSizeGroup *size_group;
 	guint i;
@@ -799,11 +898,13 @@ pdm_dialog_init (PdmDialog *dialog)
 	PdmActionInfo *cookie_sites;
 	PdmActionInfo *image_sites;
 	PdmActionInfo *password_sites;
+	PdmActionInfo *popup_sites;
 	GtkWidget *cookies_tv;
 	GtkWidget *passwords_tv;
 	GtkWidget *cookie_sites_tv;
 	GtkWidget *image_sites_tv;
 	GtkWidget *password_sites_tv;
+	GtkWidget *popup_sites_tv;
 	
 	shell = galeon_shell_get_embed_shell (galeon_shell);
 	
@@ -823,6 +924,7 @@ pdm_dialog_init (PdmDialog *dialog)
 	cookie_sites_tv = setup_cookie_sites_treeview (dialog);
 	image_sites_tv = setup_image_sites_treeview(dialog);
 	password_sites_tv = setup_password_sites_treeview(dialog);
+	popup_sites_tv = setup_popup_sites_treeview(dialog);
 
 	cookies = g_new0 (PdmActionInfo, 1);
 	galeon_embed_shell_list_cookies (shell, &cookies->list);
@@ -883,11 +985,24 @@ pdm_dialog_init (PdmDialog *dialog)
 	image_sites->data_col	= COL_IMAGE_SITES_DATA;
 	setup_action(image_sites);
 
+	popup_sites = g_new0(PdmActionInfo, 1);
+	galeon_embed_shell_list_permissions(shell, POPUPS_PERMISSION,
+					    &popup_sites->list);
+	popup_sites->dialog	= dialog;
+	popup_sites->remove_id	= PROP_POPUP_SITES_REMOVE;
+	popup_sites->add	= pdm_dialog_popup_site_add;
+	popup_sites->remove	= pdm_dialog_popup_site_remove;
+	popup_sites->free	= pdm_dialog_site_permissions_free;
+	popup_sites->treeview	= popup_sites_tv;
+	popup_sites->data_col	= COL_POPUP_SITES_DATA;
+	setup_action(popup_sites);
+
 	dialog->priv->cookies = cookies;
 	dialog->priv->passwords = passwords;
 	dialog->priv->password_sites = password_sites;
 	dialog->priv->cookie_sites = cookie_sites;
 	dialog->priv->image_sites = image_sites;
+	dialog->priv->popup_sites = popup_sites;
 }
 
 static void
@@ -902,6 +1017,7 @@ pdm_dialog_finalize (GObject *object)
 
         g_return_if_fail (dialog->priv != NULL);
 
+	pdm_dialog_site_permissions_free (dialog->priv->popup_sites, NULL);
 	pdm_dialog_site_permissions_free (dialog->priv->cookie_sites, NULL);
 	pdm_dialog_site_permissions_free (dialog->priv->image_sites, NULL);
 	pdm_dialog_passwords_free (dialog->priv->passwords, NULL);
