@@ -109,6 +109,41 @@ void popup_cmd_image_in_new_window (BonoboUIComponent *uic,
 			      GALEON_NEW_TAB_IN_NEW_WINDOW);
 }
 
+void popup_cmd_image_block_site(BonoboUIComponent *uic,
+				GaleonEmbedPopup *popup,
+				const char* verbname)
+{
+	GaleonEmbedEvent *info;
+	GaleonEmbedShell *embed_shell;
+	const GValue *value;
+	PermissionActionType perm_action;
+
+	embed_shell = galeon_shell_get_embed_shell(galeon_shell);
+	
+	info = galeon_embed_popup_get_event(popup);
+	
+	galeon_embed_event_get_property(info, "image", &value);
+
+	galeon_embed_shell_test_permission(embed_shell,
+					   g_value_get_string(value),
+					   IMAGES_PERMISSION,
+					   &perm_action);
+	if (perm_action == DENY_ACTION)
+	{
+		galeon_embed_shell_set_permission(embed_shell,
+						  g_value_get_string(value),
+						  IMAGES_PERMISSION,
+						  TRUE);
+	}
+	else
+	{
+		galeon_embed_shell_set_permission(embed_shell,
+						  g_value_get_string(value),
+						  IMAGES_PERMISSION,
+						  FALSE);
+	}
+}
+
 void popup_cmd_add_bookmark (BonoboUIComponent *uic, 
 			     GaleonEmbedPopup *popup, 
 			     const char* verbname)
