@@ -59,6 +59,7 @@
 	"go=go_button;" \
 	"zoom=zoom;" \
 	"spinner=spinner;" \
+	"fullscreen=fullscreen_button;" \
 	"separator;"
 
 
@@ -456,8 +457,7 @@ window_cmd_file_close_tab (BonoboUIComponent *uic,
 	
 	tab = galeon_window_get_active_tab (window);
 	g_return_if_fail (tab != NULL);
-
-	galeon_window_remove_tab (window, tab);
+	galeon_tab_destroy (tab);
 }
 
 void 
