@@ -27,6 +27,9 @@
 #include <libgnomevfs/gnome-vfs-mime.h>
 #include <libxml/parser.h>
 
+/* precomputed g_unichar_to_utf8 (8230, ellipsis); */
+#define ELLIPSIS "\xe2\x80\xa6"
+
 /* local prototypes */
 
 char *
@@ -161,34 +164,33 @@ gul_string_escape_uline_accel (const gchar *text)
 gchar *
 gul_string_shorten (const gchar *str, gint target_length)
 {
-	gchar *new_str;
+	GString *new_str;
+	gchar *ret;
 	gint actual_length, first_length, second_length;
 
 	if (!str) return NULL;
 
-	actual_length = strlen (str);
+	actual_length = g_utf8_strlen (str, -1);
 
-	/* if the string is already short enough, or if it's too short for
-	 * us to shorten it, return a new copy */
-	if (actual_length <= target_length ||
-	    actual_length <= 3)
+	/* if the string is already short enough return a new copy */
+	if (actual_length <= target_length)
 		return g_strdup (str);
 
-	/* allocate new string */
-	new_str = g_new (gchar, target_length + 1);
-
-	/* calc lengths to take from beginning and ending of str */
-	second_length = (target_length - 3) / 2;
-	first_length = target_length - 3 - second_length;
+	/* calc lengths to take from beginning and ending of str
+	 * (1 == ellipsis is only one character) */
+	second_length = (target_length - 1) / 2;
+	first_length = target_length - 1 - second_length;
 
 	/* create string */
-	strncpy (new_str, str, first_length);
-	strncpy (new_str + first_length, "...", 3);
-	strncpy (new_str + first_length + 3,
-		 str + actual_length - second_length, second_length);
-	new_str[target_length] = '\0';
+	new_str = g_string_sized_new (target_length);
+	g_string_append_len (new_str, str, first_length);
+	g_string_append (new_str, ELLIPSIS);
+	g_string_append (new_str, g_utf8_offset_to_pointer (str, actual_length - second_length));
 
-	return new_str;
+	ret = new_str->str;
+	g_string_free (new_str, FALSE);
+
+	return ret;
 }
 
 /**
